module UTILS

use PARAM
use GLOBALS

implicit none
CONTAINS 

function UTIL(cx)
real(long) , intent(in) :: cx
real(long) :: UTIL

if (flag_CRRA==0) then
    UTIL = log(cx)
else
    UTIL = (cx**(1-sigma))/(1-sigma)
endif
endfunction UTIL

function UINV(UX)
real(long) , intent(in) :: UX
real(long) :: UINV
integer:: readum

if (flag_CRRA==0) then
    UINV = exp(UX)
else
    UINV = (UX*(1-sigma))**(1/(1-sigma))
endif
endfunction UINV

function UINV_PRIME(UX)
real(long) , intent(in) :: UX
real(long) :: UINV_PRIME
integer:: readum

if (flag_CRRA==0) then
    UINV_PRIME = exp(UX)
else
    UINV_PRIME = (UX*(1-sigma))**(sigma/(1-sigma))
endif
endfunction UINV_PRIME

function kapa_precision(precision)
real(long) , intent(in) :: precision
real(long) :: kapa_precision
kapa_precision = precision_cost*(precision**1.4)
endfunction kapa_precision

function pi_a(effort_in)
real(long) , intent(in) :: effort_in
real(long) :: pi_a

if (fixed_effort==1) then 
    pi_a = max_pi_possible*e**(1/pi_curvature)/(max_effort**(1/pi_curvature))
else
    pi_a = max_pi_possible*effort_in**(1/pi_curvature)/(max_effort**(1/pi_curvature))
endif
endfunction pi_a

function pi_a_prime(effort_in)
real(long) , intent(in) :: effort_in
real(long) :: pi_a_prime

if (fixed_effort==1) then 
    pi_a_prime = max_pi_possible*(1/pi_curvature)*e**(1/pi_curvature-1)/(max_effort**(1/pi_curvature))
else
    pi_a_prime = max_pi_possible*(1/pi_curvature)*effort_in**(1/pi_curvature-1)/(max_effort**(1/pi_curvature))
endif
endfunction pi_a_prime

function pi_a_double_prime(effort_in)
real(long) , intent(in) :: effort_in
real(long) :: pi_a_double_prime

pi_a_double_prime = max_pi_possible*(1/pi_curvature)*(1/pi_curvature-1)*effort_in**(1/pi_curvature-1-1)/(max_effort**(1/pi_curvature))
endfunction pi_a_double_prime

function theta_a(effort_in,gamma)
real(long) , intent(in) :: effort_in, gamma
real(long) :: theta_a, gamma_plus

gamma_plus = 1.0
theta_a = (max_precision-gamma)+(effort_in-min_effort)/(max_effort-min_effort)*(gamma)
theta_a = max_precision-gamma*((max_effort-effort_in)/(max_effort-min_effort))**gamma_plus
theta_a = factor_effectivness_theta*theta_a_parameter
endfunction theta_a


function theta_a_prime(effort_in,gamma)
real(long) , intent(in) :: effort_in, gamma
real(long) :: theta_a_prime, gamma_plus

gamma_plus = 1.0
theta_a_prime = 1/(max_effort-min_effort)*(gamma);
theta_a_prime = gamma*gamma_plus*((max_effort-effort_in)/(max_effort-min_effort))**gamma_plus/(max_effort-effort_in)
theta_a_prime = 10*theta_a_prime
theta_a_prime = efficiency_factor_signal_slope*gamma
endfunction theta_a_prime

function stdev(x,n)

integer, intent(in)  :: n 
integer :: i
real (long) :: aver, sumxx, dev, stdev 
real(long), dimension(n) , intent(in) :: x
aver = 0.0 
sumxx = 0.0 
do i = 1, n 
  dev = x(i) - aver 
  aver = aver + dev/i 
  sumxx = sumxx + dev*(x(i)-aver) 
end do 
stdev = SQRT( sumxx / (n-1) ) 
endfunction stdev

endmodule UTILS